;;
;; Cisco mode by 
;; version 0.2
;;
;; Changelog
;;          0.2:   added banner motd fontifying
;;
;;          0.1:   initial version, some stuff stolen from
;;                 cperl-mode.el
;;
;; Credits:
;;          Maintainer: Thomas Linden <tom@daemon.de>
;;          Help with regexp stuff: Jens Heunemann
;;
;; Features: Syntay highlighting and imenu entries
;;

(defvar cisco-major-face       'cisco-major-face "")
(defvar cisco-major-name-face  'cisco-major-name-face "")
(defvar cisco-no-face          'cisco-no-face "")
(defvar cisco-string-face      'cisco-string-face "")
(defvar cisco-ware-face        'cisco-ware-face "")
(defvar cisco-setting-face     'cisco-setting-face "")
(defvar cisco-wrong-face       'cisco-wrong-face "")
(defvar cisco-name-face        'cisco-name-face "")

(defface cisco-major-face
  '(
   (((class color) (background light)) (:foreground "Blue"          :background "lightyellow2" :bold t))
   (((class color) (background dark))  (:foreground "LightSkyBlue"  :background "Blue"         :bold t))
   (t (:inverse-video t :bold t))
   )
  "Font Lock mode face used to highlight major keywords."
  )

(defface cisco-major-name-face
  '(
   (((class color) (background light)) (:foreground "Blue"          :background "lightyellow2"))
   (((class color) (background dark))  (:foreground "LightSkyBlue"  :background "Blue"        ))
   (t (:inverse-video t :bold t))
   )
  "Font Lock mode face used to highlight major names."
  )

(defface cisco-no-face
  '(
   (((class color) (background light)) (:background "FloralWhite" :italic t))
   (((class color) (background dark))  (:background "Blue" :italic t))
   (t (:inverse-video t))
   )
  "Font Lock mode face used to highlight no commands."
  )

(defface cisco-string-face
  '(
   (((class color) (background light)) (:foreground "Red" :italic t))
   (((class color) (background dark))  (:foreground "Red" :italic t))
   (t (:inverse-video t))
   )
  "Font Lock mode face used to highlight descriptive strings."
  )

(defface cisco-ware-face
  '(
   (((class color) (background light)) (:foreground "Coral"    :background "FloralWhite" :underline t :bold t))
   (((class color) (background dark))  (:foreground "FloralWhite"  :background "DarkGreen"   :underline t))
   (t (:inverse-video t :underline t))
   )
  "Font Lock mode face used to highlight soft/firmware names."
  )


(defface cisco-setting-face
  '(
   (((class color) (background light)) (:foreground "CadetBlue"    :bold t))
   (((class color) (background dark))  (:foreground "CadetBlue"    :bold t))
   (t (:inverse-video t :underline t))
   )
  "Font Lock mode face used to highlight time values/defines."
  )


(defface cisco-wrong-face
  '(
   (((class color) (background light)) (:background "IndianRed" :foreground "White" :bold t))
   (((class color) (background dark))  (:foreground "Red"       :bold t))
   (t (:inverse-video t :underline t))
   )
  "Font Lock mode face used to highlight wrong commands."
  )

(defface cisco-name-face
  '(
   (((class color) (background light)) (:foreground "DarkSlateGray"))
   (((class color) (background dark))  (:foreground "BlueViolet"   ))
   (t (:inverse-video t :underline t))
   )
  "Font Lock mode face used to highlight names."
  )

(defvar cisco-address-face     ; 172.16.1.1
  font-lock-type-face          "Highlight IP addresses and netmasks")
(defvar cisco-comment-face     ; ! ...
  font-lock-comment-face       "Highlight comments")
(defvar cisco-minor-face       ; address, route, mode, authentication
  font-lock-keyword-face       "Highlight subcommands")
(defvar cisco-flag-face        ; permit, log, in
  font-lock-variable-name-face "Highlight command flags")
(defvar cisco-value-face       ; 200, 12, 222
  font-lock-function-name-face "Highlight numerical values")
;(defvar cisco-name-face        ; ... ...
;  font-lock-constant-face      "Highlight names")
(defvar cisco-attention-face       ; interface ...
  font-lock-warning-face       "Highlight block beginnings")



(defvar cisco-mode-syntax-table nil "Syntax table for cisco-mode buffers.")
(defvar cisco-mode-map nil "Keymap used in cisco mode.")
(defvar cisco-mode-abbrev-table nil "")

(defvar cisco-imenu-expression
  '(
    ("Interfaces"     "^ *\\(interface[ ]*.*\\)!*" 1 )
    ("Controllers"    "^ *\\(controller[ ]*.*\\)!*" 1 )
    ("Router entries" "^ *\\(router[ ]*.*\\)!*" 1 )
    ("Lines"          "^ *\\(line[ ]*.*\\)!*" 1 )
    ("Spe's"          "^ *\\(spe[ ]*.*\\)!*" 1 )
    ("Modems"         "^ *\\(modem\\-pool[ ]*.*\\)!*" 1 )
    )
  "Imenu generic expression for CISCO mode.  See `imenu-generic-expression'.")


;;;
;;; Syntax highlighting definitions, may become huge regexps, beware!
;;;
(defvar cisco-font-lock-keywords 
   (list
    
    '(" \\(full\\|ppp\\|v120\\|v110\\|chap\\|pap\\|in\\|out\\|deny\\|permit\\|log\\|eq\\|network\\|ne\\|range\\|default\\|public\\|all\\|mica\\|primary\\-net5\\|passive\\|aktive\\|mppc\\|dedicated\\|line\\|aaa\\|if\\-needed\\|localtime\\|local\\|tacacs\\+\\|primary\\|flash\\|datetime\\|debug\\|uptime\\|login\\|group\\|start\\-stop\\|any\\|none\\|RO\\|RW\\|max\\-conn\\|dns\\-server\\|subnet\\-zero\\|routing\\)\\>" 
      1 cisco-flag-face t
      )

    '(" \\([0-9-\. ,]+\\)" 1 cisco-value-face t)

    '("access-list +\\([0-9]+\\) +" 1 cisco-attention-face t)

    '(
      " \\(tcp\\-keepalives\\-in\\|host\\|type\\|address\\|unnumbered\\|mroute\\-cache\\|timeslots\\|timestamps\\|tcp\\|udp\\|icmp\\|route\\|switch\\-type\\|incoming\\-voice\\|idle\\-timeout\\|encapsulation\\|in\\-band\\|mode\\|dynamic\\|authentication\\|authorization\\|accounting\\|ip\\|log\\|community\\|motd\\|engineID\\|local\\|input\\|system\\-shutdown\\|synchronous\\|server\\|Dialin\\|autoconfigure\\|location\\|clock\\-period\\|update\\-calendar\\|modem\\|header\\-compression\\|source\\|virtual\\-template\\|timezone\\|disable\\|secret\\|system\\|password\\-encryption\\|password\\|classless\\|static\\|pool\\-range\\|called\\-number\\|async\\-bootp\\|new\\-model\\|interval\\)\\>"
      1 cisco-name-face t
      )

    '("^ *\\(end\\|ip\\|ppp\\|modem\\|dialer\\|ntp\\|network\\|async\\-bootp\\|async\\|peer\\|description\\|isdn\\|access\\-list\\|group\\-range\\|hold\\-range\\|hold\\-queue\\|encapsulation\\|keepalive\\|snmp\\-server\\|tacacs\\-server\\|banner\\|ntp\\|scheduler\\|transport\\|access\\-class\\|access\\-group\\|logging\\|login\\|cdp\\|snmp\\|compress\\|duplex\\|autodetect\\|pri\\-group\\|shutdown\\|fair\\-queue\\|clock\\|virtual\\-profile\\|aaa\\|username\\|password\\|resource\\-pool\\|enable\\|hostname\\|boot\\|service\\|version\\|redistribute\\|firmware\\)\\>"
      1 cisco-minor-face t)


    '("\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)"
      1 cisco-address-face t )

    '("\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)  *\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)"
      (1 cisco-address-face t ) (2 cisco-value-face t) )

    '("\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) \\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) \\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)"
      (1 cisco-address-face t ) (2 cisco-address-face t ) (3 cisco-value-face t) )

    '("\\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) \\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) \\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\) \\([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+\\)"
      (1 cisco-address-face t ) (2 cisco-value-face t) (3 cisco-address-face t ) (4 cisco-value-face t) )

    '("description +\\(.*\\)$"          1 cisco-string-face t)
    '("username +\\([0-9a-zA-Z\__]*\\)" 1 cisco-string-face t)
    '("password +[0-9]?\\(.*\\)\\>"     1 cisco-string-face t)
    '("secret +[0-9]?\\(.*\\)\\>"       1 cisco-string-face t)

    '("^ *\\(interface\\|controller\\|line\\|router\\|spe\\|modem\\-pool\\)"
      1 cisco-major-face t override t)
    '("^ *\\(interface\\|controller\\|line\\|router\\|spe\\|modem\\-pool\\)\\(.*\\)$"
      2 cisco-major-name-face t)

    '("location +\\(.*\\)$" 1 cisco-ware-face t)
    '("boot +.* +\\(.*\\)$" 1 cisco-ware-face t)

    '("clock +\\(.*\\)$" 1 cisco-setting-face t)

    '("hostname +\\(.*\\)$" 1 cisco-string-face t)

    '("\\(Current configuration:\\)" 1 cisco-wrong-face t)

    '("^.*\\(no .*\\)$"
      1 cisco-no-face t)

    '("\\`.*hallo.*hallo.*\\'" 1 cisco-string-face t)

    '("^.*\\(!.*\\)$"
      1 cisco-comment-face t)

    '("banner motd \\(.\\)\\(\\(.*\n\\)*\\)\\1"
      (2 cisco-string-face t)
      )
    )
  "Additional expressions to highlight in cisco mode.")


(defun cisco-mode ()
  "Major mode for editing CISCO configurations.
Entry to this mode calls the value of `cisco-mode-hook'
if that value is non-nil."
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'cisco-mode
        mode-name "CISCO")

  (make-local-variable 'imenu-generic-expression)
  (setq imenu-generic-expression cisco-imenu-expression)
  (setq imenu-case-fold-search nil)
  (require 'imenu)

  (set (make-local-variable 'font-lock-defaults)
       '(cisco-font-lock-keywords nil))

  (run-hooks 'cisco-mode-hook)
)

(provide 'cisco-mode) 

